/*
COPYRIGHT, LICENSE AND WARRANTY INFORMATION

This software module has been originally developed by Nokia Corporation. 

The software module is provided free of charge and made available for use 
by the Licensee, who shall be a member of the MPEG standardization 
committee, under the following conditions:

The Supplied Data may only be used for the purpose of developing, testing 
and promulgating the MPEG technology standard(s) for 3D video coding.

Nokia retains the ownership of copyrights to the Software. There is no 
patent nor other intellectual property right of Nokia licensed under this 
Statement (except the copyright license above). Licensee hereby assumes 
sole responsibility to secure any other intellectual property rights 
needed, if any. For example, if patent licenses  are required, it is 
their responsibility to acquire the license before utilizing the Software.

The license by Nokia is subject to that the Licensee grants to Nokia the 
non-exclusive, worldwide, royalty-free, perpetual and irrevocable 
covenant that the Licensee(s) shall not bring a suit before any court or 
administrative agency or otherwise assert a claim for infringement under 
the Licensee intellectual property rights that, but for a license, would 
be infringed by the Software against  
   (a)  Nokia or Nokia's Affiliate; or  
   (b)  other recipient of a license and covenant not to sue with respect 
        to the Software from Nokia; or 
   (c)  contractor, customer or distributor of a party listed above in a 
        or b,  
which suit or claim is related to the Software or use thereof.

The Licensee(s) further agrees to grant a reciprocal license to Nokia 
(as granted by Nokia to the Licensee(s) on the modifications made by 
Licensee(s) to the Software. 

THE SOFTWARE IS PROVIDED "AS IS" AND THE ORIGINAL DEVELOPER DISCLAIMS 
ANY AND ALL WARRANTIES WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING 
BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS 
FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. THOSE INTENDING TO USE 
THE SOFTWARE ARE EXPRESSLY ADVISED THAT ITS USE MAY INFRINGE EXISTING 
PATENTS AND BE SUBJECT TO ROYALTY PAYMENTS TO PATENT OWNERS. ANYONE 
USING THE SOFTWARE ON THE BASIS OF THIS LICENSE AGREES TO OBTAIN THE 
NECESSARY PERMISSIONS FROM ANY AND ALL APPLICABLE PATENT OWNERS FOR SUCH 
USE.

IN NO EVENT SHALL THE ORIGINAL DEVELOPER BE LIABLE FOR ANY CLAIM, 
DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT 
OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE 
OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

This copyright, license and warranty information notice must be retained 
in all copies and derivative works of the Software or substantial 
portions thereof.
*/
#include "global.h"
#include "memalloc.h"
#include "3dv_acquisition.h"

#define max(a,b)    (((a) > (b)) ? (a) : (b))
#define min(a,b)    (((a) < (b)) ? (a) : (b))


int32 getVarLength(int32 exponent, int32 precision) 
{
	if (exponent==0)
		return max(0, -30+precision);
	else 
		return max(0, exponent-31+precision);
}

void get_exponent_mantissa(ThreeDVAE* threeDV_ae,int voidx)
{
	double arg=fabs(threeDV_ae->original[voidx]);
	double log_val;
	int32 expo,v=0,i;
	double mant;
	int32 exponnent=0;
	int32 mantissa=0;
	int32 bits=0;

	if(threeDV_ae->pred_mode)
	{
		int32 bit;
		int mant_prec=threeDV_ae->precision;
		/* Find expo & mant when arg=(-1)^sign*2^expo*(0.mant),*/
		if (arg==0.0)
		{
			expo=0;
			mant=0;
		} 
		else 
		{
			log_val= log(arg)/log(2.0);
			expo = (int)floor(log_val);
			mant = pow(2.0, log_val - expo);
			if (fabs(mant) > 1.0) 
			{
				mant = mant / (double)2.0;
				expo= expo + 1;
			}		
		}

		/* Convert expo & mant so that X=(-1)^s*2^expo*(1.mant) */
		if (expo == 0 && mant ==0.0 ) 
		{
			v=0;
		} 
		else if (expo> -30)
		{ 
			while ((double)fabs(mant)<(double)1.0) 
			{
				mant=mant*2.0;
				expo=expo-1;
			}
			if (mant>=0.0)
				mant = mant-1.0;
			v= expo + mant_prec; // number of necessary mantissa bits in the case of truncation
			if (v<0)
				v=0;
			expo += 31;
		} 
		else if (expo == -30) 
		{
			v= expo + mant_prec; // number of necessary mantissa bits in the case of truncation
			if (v<0) 
				v=0;
			expo=0;
		}

		exponnent = expo;
		if(v>30)
			v=30;

		/* Convert a float number mant (0<=mant<1) into a binary representation N which is a mantissa with v bits */
		mantissa=0;


		for (i=1; i<=v; i++ )
		{
			bit = (2.0*mant >= 1.0);
			mant = 2.0*mant - (double)bit;
			mantissa = (mantissa << 1 ) | bit ; // MSB of M corresponds to 1/2 and MSB-1 to 1/4 and so on.
		}
		bits=v;
	}
	else
	{
		int32 mantissa_len=threeDV_ae->mantissa_length;
		log_val= log(arg)/log(2.0);
		expo = (int)floor(log_val);
		exponnent=expo+31;

		mantissa=floor((arg/pow(2,expo)-1)*pow(2,mantissa_len)+0.5);
		bits=mantissa_len;
	}
	threeDV_ae->element[voidx].exponent=exponnent;
	threeDV_ae->element[voidx].mantissa=mantissa;
	threeDV_ae->element[voidx].mantissa_bits=bits;
}
void get_rec_double_type(ThreeDVAE* threeDV_ae, int voidx)
{
	int32 i;
	int32 sgn= (threeDV_ae->element[voidx].sign==0) ? 1 : -1;
	int32 exponent=threeDV_ae->element[voidx].exponent;
	int32 mantissa=threeDV_ae->element[voidx].mantissa;
	int32 exponent_size=(1<<threeDV_ae->exponent_size)-1;
	double *recon=&(threeDV_ae->rec[voidx]);

	assert(exponent!=exponent_size);

	if(threeDV_ae->pred_mode)
	{
		int32 length=getVarLength(exponent,threeDV_ae->precision);


		double factor=1.0;

		if(length>30)
			length=30;
		for(i=0;i<length;i++)
			factor=factor/2;


		*recon=0.0;



		for (i=0;i<(int)length;i++) 
		{
			*recon += factor*((mantissa>>i)&0x01);
			factor *= (double)2.0;
		}
		if (exponent>0 && exponent<exponent_size)
			*recon = (double)sgn*(double)pow((double)2,(double)exponent-31)*(1.0+*recon);
		if (exponent==0)
			*recon = (double)sgn*(double)pow((double)2,(double)-30)*(*recon);	
	}
	else
	{
		int32 mantissa_len=threeDV_ae->mantissa_length;
		*recon=sgn*pow(2,exponent-31)*(1+mantissa/pow(2,mantissa_len));
	}
}
int get_mem_acquisition_info(ThreeDVAcquisitionInfo** threeDV_acquisition_info)
{
	*threeDV_acquisition_info=calloc(1,sizeof(ThreeDVAcquisitionInfo));
	if(*threeDV_acquisition_info==NULL)
		no_mem_exit("get_mem_acquisition_info:*threeDV_acquisition_info");
	(*threeDV_acquisition_info)->focal_length_x_ae=calloc(1,sizeof(ThreeDVAE));
	if((*threeDV_acquisition_info)->focal_length_x_ae==NULL)
		no_mem_exit("get_mem_acquisition_info:(*threeDV_acquisition_info)->focal_length_x_ae");
	(*threeDV_acquisition_info)->focal_length_y_ae=calloc(1,sizeof(ThreeDVAE));
	if((*threeDV_acquisition_info)->focal_length_y_ae==NULL)
		no_mem_exit("get_mem_acquisition_info:(*threeDV_acquisition_info)->focal_length_y_ae");

	(*threeDV_acquisition_info)->principal_point_x_ae=calloc(1,sizeof(ThreeDVAE));
	if((*threeDV_acquisition_info)->principal_point_x_ae==NULL)
		no_mem_exit("get_mem_acquisition_info:(*threeDV_acquisition_info)->principal_point_x_ae");
	(*threeDV_acquisition_info)->principal_point_y_ae=calloc(1,sizeof(ThreeDVAE));
	if((*threeDV_acquisition_info)->principal_point_y_ae==NULL)
		no_mem_exit("get_mem_acquisition_info:(*threeDV_acquisition_info)->principal_point_y_ae");

	(*threeDV_acquisition_info)->translation_ae=calloc(1,sizeof(ThreeDVAE));
	if((*threeDV_acquisition_info)->translation_ae==NULL)
		no_mem_exit("get_mem_acquisition_infO:(*threeDV_acquisition_info)->translation_ae");

	(*threeDV_acquisition_info)->depth_near_ae=calloc(1,sizeof(ThreeDVAE));
	if((*threeDV_acquisition_info)->depth_near_ae==NULL)
		no_mem_exit("get_mem_acquisition_infO:(*threeDV_acquisition_info)->depth_near_ae");
	(*threeDV_acquisition_info)->depth_far_ae=calloc(1,sizeof(ThreeDVAE));
	if((*threeDV_acquisition_info)->depth_far_ae==NULL)
		no_mem_exit("get_mem_acquisition_infO:(*threeDV_acquisition_info)->depth_far_ae");
	return sizeof(ThreeDVAcquisitionInfo)+7*sizeof(ThreeDVAE);
}

void init_acquisition_info(ThreeDVAcquisitionInfo* ThreeDV_acquisition_info)
{
	memset(ThreeDV_acquisition_info->focal_length_x_ae,0x00,sizeof(ThreeDVAE));
	memset(ThreeDV_acquisition_info->focal_length_y_ae,0x00,sizeof(ThreeDVAE));
	memset(ThreeDV_acquisition_info->principal_point_x_ae,0x00,sizeof(ThreeDVAE));
	memset(ThreeDV_acquisition_info->principal_point_y_ae,0x00,sizeof(ThreeDVAE));
	memset(ThreeDV_acquisition_info->translation_ae,0x00,sizeof(ThreeDVAE));
	memset(ThreeDV_acquisition_info->depth_near_ae,0x00,sizeof(ThreeDVAE));
	memset(ThreeDV_acquisition_info->depth_far_ae,0x00,sizeof(ThreeDVAE));
	ThreeDV_acquisition_info->focal_length_x_ae->pred_mode=1;
	ThreeDV_acquisition_info->focal_length_x_ae->delta_flag=0;
	ThreeDV_acquisition_info->focal_length_x_ae->exponent_size=6;

	ThreeDV_acquisition_info->focal_length_y_ae->pred_mode=1;
	ThreeDV_acquisition_info->focal_length_y_ae->delta_flag=0;
	ThreeDV_acquisition_info->focal_length_y_ae->exponent_size=6;

	ThreeDV_acquisition_info->principal_point_x_ae->pred_mode=1;
	ThreeDV_acquisition_info->principal_point_x_ae->delta_flag=0;
	ThreeDV_acquisition_info->principal_point_x_ae->exponent_size=6;

	ThreeDV_acquisition_info->principal_point_y_ae->pred_mode=1;
	ThreeDV_acquisition_info->principal_point_y_ae->delta_flag=0;
	ThreeDV_acquisition_info->principal_point_y_ae->exponent_size=6;

	ThreeDV_acquisition_info->translation_ae->pred_mode=1;
	ThreeDV_acquisition_info->translation_ae->delta_flag=1;
	ThreeDV_acquisition_info->translation_ae->exponent_size=6;

	ThreeDV_acquisition_info->depth_near_ae->pred_mode=0;
	ThreeDV_acquisition_info->depth_near_ae->delta_flag=0;
	ThreeDV_acquisition_info->depth_near_ae->exponent_size=7;

	ThreeDV_acquisition_info->depth_far_ae->pred_mode=0;
	ThreeDV_acquisition_info->depth_far_ae->delta_flag=0;
	ThreeDV_acquisition_info->depth_far_ae->exponent_size=7;

}

void copy_acquisition_info(ThreeDVAcquisitionInfo* dest_acquisition_infor, ThreeDVAcquisitionInfo* src_acquisition_info)
{
	memcpy(dest_acquisition_infor->focal_length_x_ae,src_acquisition_info->focal_length_x_ae,sizeof(ThreeDVAE));
	memcpy(dest_acquisition_infor->focal_length_y_ae,src_acquisition_info->focal_length_y_ae,sizeof(ThreeDVAE));
	memcpy(dest_acquisition_infor->principal_point_x_ae,src_acquisition_info->principal_point_x_ae,sizeof(ThreeDVAE));
	memcpy(dest_acquisition_infor->principal_point_y_ae,src_acquisition_info->principal_point_y_ae,sizeof(ThreeDVAE));
	memcpy(dest_acquisition_infor->translation_ae,src_acquisition_info->translation_ae,sizeof(ThreeDVAE));
	memcpy(dest_acquisition_infor->depth_near_ae,src_acquisition_info->depth_near_ae,sizeof(ThreeDVAE));
	memcpy(dest_acquisition_infor->depth_far_ae,src_acquisition_info->depth_far_ae,sizeof(ThreeDVAE));
}

/*!
************************************************************************
* \brief
*    free the memory for 3DV acquisition information
************************************************************************
*/
void free_mem_acquisition_info(ThreeDVAcquisitionInfo* threeDV_acqusition_info)
{
	if(threeDV_acqusition_info)
	{
		free(threeDV_acqusition_info->focal_length_x_ae);
		free(threeDV_acqusition_info->focal_length_y_ae);
		free(threeDV_acqusition_info->principal_point_x_ae);
		free(threeDV_acqusition_info->principal_point_y_ae);
		free(threeDV_acqusition_info->translation_ae);
		free(threeDV_acqusition_info->depth_near_ae);
		free(threeDV_acqusition_info->depth_far_ae);
		free(threeDV_acqusition_info);
	}
}
#if SONY_MDFY_SEI_AQINF_M24730
void get_exponent_mantissa_SEI(ThreeDVAESEI* threeDV_ae)
{
	double arg=fabs(threeDV_ae->original);
	double log_val;
	int32 expo,v=0,i;
	double mant;
	int32 exponnent=0;
	int32 mantissa=0;
	int32 bits=0;

	if(threeDV_ae->pred_mode)
	{
		int32 bit;
		int mant_prec=threeDV_ae->precision;
		/* Find expo & mant when arg=(-1)^sign*2^expo*(0.mant),*/
		if (arg==0.0)
		{
			expo=0;
			mant=0;
		} 
		else 
		{
			log_val= log(arg)/log(2.0);
			expo = (int)floor(log_val);
			mant = pow(2.0, log_val - expo);
			if (fabs(mant) > 1.0) 
			{
				mant = mant / (double)2.0;
				expo= expo + 1;
			}		
		}

		/* Convert expo & mant so that X=(-1)^s*2^expo*(1.mant) */
		if (expo == 0 && mant ==0.0 ) 
		{
			v=0;
		} 
		else if (expo> -30)
		{ 
			while ((double)fabs(mant)<(double)1.0) 
			{
				mant=mant*2.0;
				expo=expo-1;
			}
			if (mant>=0.0)
				mant = mant-1.0;
			v= expo + mant_prec; // number of necessary mantissa bits in the case of truncation
			if (v<0)
				v=0;
			expo += 31;
		} 
		else if (expo == -30) 
		{
			v= expo + mant_prec; // number of necessary mantissa bits in the case of truncation
			if (v<0) 
				v=0;
			expo=0;
		}

		exponnent = expo;
		if(v>30)
			v=30;

		/* Convert a float number mant (0<=mant<1) into a binary representation N which is a mantissa with v bits */
		mantissa=0;


		for (i=1; i<=v; i++ )
		{
			bit = (2.0*mant >= 1.0);
			mant = 2.0*mant - (double)bit;
			mantissa = (mantissa << 1 ) | bit ; // MSB of M corresponds to 1/2 and MSB-1 to 1/4 and so on.
		}
		bits=v;
	}
	else
	{
		int32 mantissa_len=threeDV_ae->mantissa_length;
		log_val= log(arg)/log(2.0);
		expo = (int)floor(log_val);
		exponnent=expo+31;

		mantissa=floor((arg/pow(2,expo)-1)*pow(2,mantissa_len)+0.5);
		bits=mantissa_len;
	}
	threeDV_ae->element.exponent=exponnent;
	threeDV_ae->element.mantissa=mantissa;
	threeDV_ae->element.mantissa_bits=bits;
}
void get_rec_double_type_SEI(ThreeDVAESEI* threeDV_ae)
{
	int32 i;
	int32 sgn= (threeDV_ae->element.sign==0) ? 1 : -1;
	int32 exponent=threeDV_ae->element.exponent;
	int32 mantissa=threeDV_ae->element.mantissa;
	int32 exponent_size=(1<<threeDV_ae->exponent_size)-1;
	double *recon=&(threeDV_ae->rec);

	assert(exponent!=exponent_size);

	if(threeDV_ae->pred_mode)
	{
		int32 length=getVarLength(exponent,threeDV_ae->precision);


		double factor=1.0;

		if(length>30)
			length=30;
		for(i=0;i<length;i++)
			factor=factor/2;


		*recon=0.0;



		for (i=0;i<(int)length;i++) 
		{
			*recon += factor*((mantissa>>i)&0x01);
			factor *= (double)2.0;
		}
		if (exponent>0 && exponent<exponent_size)
			*recon = (double)sgn*(double)pow((double)2,(double)exponent-31)*(1.0+*recon);
		if (exponent==0)
			*recon = (double)sgn*(double)pow((double)2,(double)-30)*(*recon);	
	}
	else
	{
		int32 mantissa_len=threeDV_ae->mantissa_length;
		*recon=sgn*pow(2,exponent-31)*(1+mantissa/pow(2,mantissa_len));
	}
}
int get_mem_acquisition_info_SEI(ThreeDVAcquisitionInfoSEI*** threeDV_acquisition_info, int num_frames)
{
	int i;

	*threeDV_acquisition_info=(ThreeDVAcquisitionInfoSEI**)malloc(num_frames*sizeof(ThreeDVAcquisitionInfoSEI*));
	if(*threeDV_acquisition_info==NULL)
		no_mem_exit("get_mem_acquisition_info:*threeDV_acquisition_info");
	for(i=0;i<num_frames;i++){
		(*threeDV_acquisition_info)[i]=(ThreeDVAcquisitionInfoSEI*)calloc(1,sizeof(ThreeDVAcquisitionInfoSEI));
	}

	for(i=0;i<num_frames;i++){
		(*threeDV_acquisition_info)[i]->focal_length_x_ae=calloc(1,sizeof(ThreeDVAESEI));
		if((*threeDV_acquisition_info)[i]->focal_length_x_ae==NULL)
		  no_mem_exit("get_mem_acquisition_info:(*threeDV_acquisition_info)->focal_length_x_ae");
		(*threeDV_acquisition_info)[i]->focal_length_y_ae=calloc(1,sizeof(ThreeDVAESEI));
		if((*threeDV_acquisition_info)[i]->focal_length_y_ae==NULL)
			no_mem_exit("get_mem_acquisition_info:(*threeDV_acquisition_info)->focal_length_y_ae");

		(*threeDV_acquisition_info)[i]->principal_point_x_ae=calloc(1,sizeof(ThreeDVAESEI));
		if((*threeDV_acquisition_info)[i]->principal_point_x_ae==NULL)
		  no_mem_exit("get_mem_acquisition_info:(*threeDV_acquisition_info)->principal_point_x_ae");
		(*threeDV_acquisition_info)[i]->principal_point_y_ae=calloc(1,sizeof(ThreeDVAESEI));
		if((*threeDV_acquisition_info)[i]->principal_point_y_ae==NULL)
		  no_mem_exit("get_mem_acquisition_info:(*threeDV_acquisition_info)->principal_point_y_ae");

		(*threeDV_acquisition_info)[i]->translation_ae=calloc(1,sizeof(ThreeDVAESEI));
		if((*threeDV_acquisition_info)[i]->translation_ae==NULL)
		  no_mem_exit("get_mem_acquisition_infO:(*threeDV_acquisition_info)->translation_ae");

		(*threeDV_acquisition_info)[i]->depth_near_ae=calloc(1,sizeof(ThreeDVAESEI));
		if((*threeDV_acquisition_info)[i]->depth_near_ae==NULL)
			no_mem_exit("get_mem_acquisition_infO:(*threeDV_acquisition_info)->depth_near_ae");
		(*threeDV_acquisition_info)[i]->depth_far_ae=calloc(1,sizeof(ThreeDVAESEI));
		if((*threeDV_acquisition_info)[i]->depth_far_ae==NULL)
			no_mem_exit("get_mem_acquisition_infO:(*threeDV_acquisition_info)->depth_far_ae");

		(*threeDV_acquisition_info)[i]->d_max_ae=calloc(1,sizeof(ThreeDVAESEI));
		if((*threeDV_acquisition_info)[i]->d_max_ae==NULL)
			no_mem_exit("get_mem_acquisition_infO:(*threeDV_acquisition_info)->d_max_ae");
		(*threeDV_acquisition_info)[i]->d_min_ae=calloc(1,sizeof(ThreeDVAESEI));
		if((*threeDV_acquisition_info)[i]->d_min_ae==NULL)
			no_mem_exit("get_mem_acquisition_infO:(*threeDV_acquisition_info)->d_min_ae");

#if NTT_MDFY_SEI_DAQINF_A0140
		(*threeDV_acquisition_info)[i]->rotation_ae = calloc(1, sizeof(ThreeDVAESEI) );
		if((*threeDV_acquisition_info)[i]->rotation_ae==NULL)
			no_mem_exit("get_mem_acquisition_infO:(*threeDV_acquisition_info)->rotation_ae");
#endif
	}
#if NTT_MDFY_SEI_DAQINF_A0140
	return sizeof(ThreeDVAcquisitionInfoSEI)*num_frames+sizeof(ThreeDVAcquisitionInfoSEI)+10*num_frames*sizeof(ThreeDVAESEI);
#else
	return sizeof(ThreeDVAcquisitionInfoSEI)*num_frames+sizeof(ThreeDVAcquisitionInfoSEI)+9*num_frames*sizeof(ThreeDVAESEI);
#endif
}

void init_acquisition_info_SEI(ThreeDVAcquisitionInfoSEI* ThreeDV_acquisition_info)
{
	memset(ThreeDV_acquisition_info->focal_length_x_ae,0x00,sizeof(ThreeDVAESEI));
	memset(ThreeDV_acquisition_info->focal_length_y_ae,0x00,sizeof(ThreeDVAESEI));
	memset(ThreeDV_acquisition_info->principal_point_x_ae,0x00,sizeof(ThreeDVAESEI));
	memset(ThreeDV_acquisition_info->principal_point_y_ae,0x00,sizeof(ThreeDVAESEI));
	memset(ThreeDV_acquisition_info->translation_ae,0x00,sizeof(ThreeDVAESEI));
	memset(ThreeDV_acquisition_info->depth_near_ae,0x00,sizeof(ThreeDVAESEI));
	memset(ThreeDV_acquisition_info->depth_far_ae,0x00,sizeof(ThreeDVAESEI));
	memset(ThreeDV_acquisition_info->d_max_ae,0x00,sizeof(ThreeDVAESEI));
	memset(ThreeDV_acquisition_info->d_min_ae,0x00,sizeof(ThreeDVAESEI));
#if NTT_MDFY_SEI_DAQINF_A0140
	memset(ThreeDV_acquisition_info->rotation_ae,0x00,sizeof(ThreeDVAESEI));
#endif

	ThreeDV_acquisition_info->focal_length_x_ae->pred_mode=1;
	ThreeDV_acquisition_info->focal_length_x_ae->exponent_size=6;

	ThreeDV_acquisition_info->focal_length_y_ae->pred_mode=1;
	ThreeDV_acquisition_info->focal_length_y_ae->exponent_size=6;

	ThreeDV_acquisition_info->principal_point_x_ae->pred_mode=1;
	ThreeDV_acquisition_info->principal_point_x_ae->exponent_size=6;

	ThreeDV_acquisition_info->principal_point_y_ae->pred_mode=1;
	ThreeDV_acquisition_info->principal_point_y_ae->exponent_size=6;

	ThreeDV_acquisition_info->translation_ae->pred_mode=1;
	ThreeDV_acquisition_info->translation_ae->exponent_size=6;

	ThreeDV_acquisition_info->depth_near_ae->pred_mode=0;
	ThreeDV_acquisition_info->depth_near_ae->exponent_size=7;

	ThreeDV_acquisition_info->depth_far_ae->pred_mode=0;
	ThreeDV_acquisition_info->depth_far_ae->exponent_size=7;

	ThreeDV_acquisition_info->d_max_ae->pred_mode=0;
	ThreeDV_acquisition_info->d_max_ae->exponent_size=7;

	ThreeDV_acquisition_info->d_min_ae->pred_mode=0;
	ThreeDV_acquisition_info->d_min_ae->exponent_size=7;

#if NTT_MDFY_SEI_DAQINF_A0140
	ThreeDV_acquisition_info->rotation_ae->pred_mode=1;
	ThreeDV_acquisition_info->rotation_ae->exponent_size=6;
#endif
}


/*!
************************************************************************
* \brief
*    free the memory for 3DV acquisition information
************************************************************************
*/
void free_mem_acquisition_info_SEI(ThreeDVAcquisitionInfoSEI* threeDV_acqusition_info)
{
	if(threeDV_acqusition_info)
	{
		free(threeDV_acqusition_info->focal_length_x_ae);
		free(threeDV_acqusition_info->focal_length_y_ae);
		free(threeDV_acqusition_info->principal_point_x_ae);
		free(threeDV_acqusition_info->principal_point_y_ae);
		free(threeDV_acqusition_info->translation_ae);
		free(threeDV_acqusition_info->depth_near_ae);
		free(threeDV_acqusition_info->depth_far_ae);
		free(threeDV_acqusition_info->d_min_ae);
		free(threeDV_acqusition_info->d_max_ae);
#if NTT_MDFY_SEI_DAQINF_A0140
		free(threeDV_acqusition_info->rotation_ae);
#endif
		free(threeDV_acqusition_info);
	}
}

#endif

