
/*!
 **************************************************************************
 * \file defines.h
 *
 * \brief
 *    Header file containing some useful global definitions
 *
 * \author
 *    Main contributors (see contributors.h for copyright, address and affiliation details)
 *     - Detlev Marpe
 *     - Karsten S?ring                 <suehring@hhi.de> 
 *     - Alexis Michael Tourapis         <alexismt@ieee.org> 
 *   
 *
 * \date
 *    21. March 2001
 **************************************************************************
 */


#ifndef _DEFINES_H_
#define _DEFINES_H_
#ifdef TRACE
#undef TRACE
#endif
#if defined _DEBUG
# define TRACE           0  //1   //!< 0:Trace off 1:Trace on 2:detailed CABAC context information
# define TRACE_RANGE     1  
#else
# define TRACE           0    //!< 0:Trace off 1:Trace on 2:detailed CABAC context information
# define TRACE_RANGE     0    
#endif

#define SMRC_3DV     1

#if(SMRC_3DV)
#include "defs_alc_common.h"
#endif

//SMRC log for analyzer
#define DECLOG4ANALYSIS  0

#define NOKIA_USTC_3DV  1

#if NOKIA_USTC_3DV
#define JM                  "17 (FRExt)"
#define VERSION             "0.4"

#define LOW_PROFILE         1     //!< 0:HP 1:EHP
#if LOW_PROFILE
#define EXT_VERSION         "LOW_PROFILE"
#else
#define EXT_VERSION         "HIGH_PROFILE"
#endif

#else
#define JM                  "17 (FRExt)"
#define VERSION             "17.2"
#define EXT_VERSION         "(FRExt)"
#endif

#define DUMP_DPB                  0    //!< Dump DPB info for debug purposes
#define PRINTREFLIST              0   //!< Print ref list info for debug purposes
#define PAIR_FIELDS_IN_OUTPUT     0    //!< Pair field pictures for output purposes
#define IMGTYPE                   0    //!< Define imgpel size type. 0 implies byte (cannot handle >8 bit depths) and 1 implies unsigned short
#define ENABLE_FIELD_CTX          1    //!< Enables Field mode related context types for CABAC
#define ENABLE_HIGH444_CTX        1    //!< Enables High 444 profile context types for CABAC. 
#define ZEROSNR                   0    //!< PSNR computation method
#define ENABLE_OUTPUT_TONEMAPPING 1    //!< enable tone map the output if tone mapping SEI present
#define JCOST_CALC_SCALEUP        1    //!< 1: J = (D<<LAMBDA_ACCURACY_BITS)+Lambda*R; 0: J = D + ((Lambda*R+Rounding)>>LAMBDA_ACCURACY_BITS)
#define DISABLE_ERC               1    //!< Disable any error concealment processes
#define JM_PARALLEL_DEBLOCK       0    //!< Enables Parallel Deblocking




#if NOKIA_USTC_3DV

//>>>tools belong to  HP & EHP
#define MVC_EXTENSION_ENABLE      0    //!< enable support for the Multiview High Profile
#define MAX_CODEVIEW              16
#define MAX_VIEWREFERENCE         2
#define REDUCED_RES               1  //!<Enable joint coding of depth and textures at different resolution

#define NOKIA_HLS_ADOPTION_M24892  1
#define LANCZOS3                  0
#define BI_LINEAR                 1
#define IMG_Y                     0
#define IMG_U                     1
#define IMG_V                     2

//<<<tools belong to  HP & EHP
#define DEPTH_PARAMETER_SET			  1      // JCT2-A0008

#if LOW_PROFILE
//>>>tools belong to  EHP
#define MTK_SKIP_DIRECT_A0045     1 //  JCT2-A0045
#define INLOOP_JVF                1
#define VISBD_VSP                 1
#define DEPTH_BASED_MVP           1
#define SLICE_HEADER_PREDICTION   1   //!<Slice header prediction from texture to depth, or depth to texture
#define IVMP                      1  //!<Inside View Motion Prediction from texture to depth
#define SONY_MDFY_SEI_AQINF_M24730   1    //!<depth_acquisition_info SEI mdfy m24730
#define NTT_MDFY_SEI_DAQINF_A0140               1   //!<depth_acquisition_info SEI modification based on JCT2-A0140
#define POZNAN_NONLINEAR_DEPTH       1      //!<Non-linear depth representation
#define POZNAN_NTT_DEPTH_REPRESENTATION_SEI     1      //!<Depth representation SEI
#define SAIT_DEPTH_INTRA_PREDICTION 1
#define SAIT_POST_DILATION_A0038     1   //!<depth dilation in post processing
#define DRWP                      1
//<<<tools belong to  EHP
#endif

#if DEPTH_BASED_MVP
#define D_MVP_MODE                  1  //!<0: complex mode 1:fast mode
#define NUM_OF_MV_GROUP              3
#define NUM_OF_INIT_CANDIDATE_MVP    8
#define SAIT_INTER_VIEW_MVP_M24824    1 //!< 1: Enable improved inter-view motion vector prediction. 0: Disable. - Jin Young Lee (Samsung&Sharp)
#define SAIT_DISP_BASED_SAD_M24824    1 //!< 1: Enable disparity-based SAD for skip/direct motion vector competitioin. 0: Disable. - Jin Young Lee
#define SAIT_SKIP_MVP_COMMENT_M24824  1 //!< Editor's comment >> 1: Average disparity for skip inter-view MV. 0: Max disaprity for skip inter-view MV.
#define SAIT_SHIFT_SAD_COMMENT_M24824 1 //!< Editor's comment >> 1: Multiplication for disparity SAD. 0: Shift operation for disparity SAD.
#if SAIT_INTER_VIEW_MVP_M24824
#define MTK_MAX_DV_SIMP_A0046          1 //  JCT2-A0046 
#endif
#endif

#if VISBD_VSP
#define MERL_VSP                       1 //!< 1: Enable adaptive reference for SKIP/DIRECT mode. 0: Disable    -Dong
#if MERL_VSP
#define SAIT_CASFP_M24819              1 //!< 1: Enable context-based adaptive skip flag positioning (CASFP) for view synthesis skip mode. 0: Disable. - Jin Young Lee
#endif
#define NOKIA_USTC_B_VSP_M26069        1
#define NOKIA_HLS_ADOPTION_M24890      1
#if REDUCED_RES
#define QC_VSP_Asymmetric_Resolution   1
#endif
#endif

#if MERL_VSP
#define DEBUG_MERL_VSP                0
#endif

#if SAIT_DEPTH_INTRA_PREDICTION
#define SAIT_DISP                     1
#define SAIT_PSIP                     1
#if SAIT_DISP
#define ETRIKHU_DEBLOCK_IN_DISP       1
#endif
#endif

#if SLICE_HEADER_PREDICTION
#define FLEXIBLE_PRED_M24946_M24981 1
#endif

#if DRWP
#define NOKIA_USTC_INT_DRWP_M26074  1
#endif

// Bug-Fix for V5
#define MTK_SH_PREDICTION_BUGFIX_V5      1    // 4. fix about memory problem
#define SAIT_B_VSP_BUGFIX_V5             1    // 5. declare missing variables 
#define SAIT_DISP_HEADER_FILE_BUGFIX_V5  1    // 6. include missing header file



#define DEBUG_MEM_LEAK                0 //!<detect probable memory leak for window platform

#endif //#if NOKIA_USTC_3DV

#if DEBUG_MEM_LEAK
#define  _CRTDBG_MAP_ALLOC
#include <stdlib.h>
#include <crtdbg.h>
#endif


#if MVC_EXTENSION_ENABLE||NOKIA_USTC_3DV
#define MVC_INIT_VIEW_ID          -1
#define BASE_VIEW_IDX             0
#define FREEPTR(ptr) { if(ptr) {free(ptr); (ptr)=NULL;} }
#endif

#include "typedefs.h"

#define SSE_MEMORY_ALIGNMENT      16

//#define MAX_NUM_SLICES 150
#define MAX_NUM_SLICES     50
#define MAX_REFERENCE_PICTURES 32               //!< H.264 allows 32 fields
#define MAX_CODED_FRAME_SIZE 8000000         //!< bytes for one frame
#define MAX_NUM_DECSLICES  16
#define MAX_DEC_THREADS    16                  //16 core deocoding;
#define MCBUF_LUMA_PAD_X        32
#define MCBUF_LUMA_PAD_Y        12
#define MCBUF_CHROMA_PAD_X      16
#define MCBUF_CHROMA_PAD_Y      8


//AVC Profile IDC definitions
typedef enum {
  FREXT_CAVLC444 = 44,       //!< YUV 4:4:4/14 "CAVLC 4:4:4"
  BASELINE       = 66,       //!< YUV 4:2:0/8  "Baseline"
  MAIN           = 77,       //!< YUV 4:2:0/8  "Main"
  EXTENDED       = 88,       //!< YUV 4:2:0/8  "Extended"
  FREXT_HP       = 100,      //!< YUV 4:2:0/8  "High"
  FREXT_Hi10P    = 110,      //!< YUV 4:2:0/10 "High 10"
  FREXT_Hi422    = 122,      //!< YUV 4:2:2/10 "High 4:2:2"
  FREXT_Hi444    = 244,      //!< YUV 4:4:4/14 "High 4:4:4"
  MVC_HIGH       = 118,      //!< YUV 4:2:0/8  "Multiview High"
  STEREO_HIGH    = 128,       //!< YUV 4:2:0/8  "Stereo High"
#if NOKIA_USTC_3DV
  ThreeDV_HIGH         = 138,
  ThreeDV_EXTEND_HIGH  =139
#endif
} ProfileIDC;

#define FILE_NAME_SIZE  255
#define INPUT_TEXT_SIZE 1024
#define MAX_VALUE       999999   //!< used for start value for some variables

#if (ENABLE_HIGH444_CTX == 1)
# define NUM_BLOCK_TYPES 22  
#else
# define NUM_BLOCK_TYPES 10
#endif


//#define _LEAKYBUCKET_

#define BLOCK_SHIFT            2
#define BLOCK_SIZE             4
#define BLOCK_SIZE_8x8         8
#define SMB_BLOCK_SIZE         8
#define BLOCK_PIXELS          16
#define MB_BLOCK_SIZE         16
#define MB_PIXELS            256 // MB_BLOCK_SIZE * MB_BLOCK_SIZE
#define MB_PIXELS_SHIFT        8 // log2(MB_BLOCK_SIZE * MB_BLOCK_SIZE)
#define MB_BLOCK_SHIFT         4
#define BLOCK_MULTIPLE         4 // (MB_BLOCK_SIZE/BLOCK_SIZE)
#define MB_BLOCK_PARTITIONS   16 // (BLOCK_MULTIPLE * BLOCK_MULTIPLE)
#define BLOCK_CONTEXT         64 // (4 * MB_BLOCK_PARTITIONS)

// These variables relate to the subpel accuracy supported by the software (1/4)
#define BLOCK_SIZE_SP      16  // BLOCK_SIZE << 2
#define BLOCK_SIZE_8x8_SP  32  // BLOCK_SIZE8x8 << 2

//  Available MB modes
typedef enum {
  PSKIP        =  0,
  BSKIP_DIRECT =  0,
  P16x16       =  1,
  P16x8        =  2,
  P8x16        =  3,
  SMB8x8       =  4,
  SMB8x4       =  5,
  SMB4x8       =  6,
  SMB4x4       =  7,
  P8x8         =  8,
  I4MB         =  9,
  I16MB        = 10,
  IBLOCK       = 11,
  SI4MB        = 12,
  I8MB         = 13,
  IPCM         = 14,
#if SAIT_DISP
  DISPMODE     = 15,
  MAXMODE      = 16
#else
  MAXMODE      = 15
#endif
} MBModeTypes;

// number of intra prediction modes
#define NO_INTRA_PMODE  9

// Direct Mode types
typedef enum {
  DIR_TEMPORAL = 0, //!< Temporal Direct Mode
  DIR_SPATIAL  = 1 //!< Spatial Direct Mode
} DirectModes;

// CAVLC block types
typedef enum {
  LUMA              =  0,
  LUMA_INTRA16x16DC =  1,
  LUMA_INTRA16x16AC =  2,
  CB                =  3,
  CB_INTRA16x16DC   =  4,
  CB_INTRA16x16AC   =  5,
  CR                =  8,
  CR_INTRA16x16DC   =  9,
  CR_INTRA16x16AC   = 10
} CAVLCBlockTypes;

// CABAC block types
typedef enum {
  LUMA_16DC     =   0,
  LUMA_16AC     =   1,
  LUMA_8x8      =   2,
  LUMA_8x4      =   3,
  LUMA_4x8      =   4,
  LUMA_4x4      =   5,
  CHROMA_DC     =   6,
  CHROMA_AC     =   7,
  CHROMA_DC_2x4 =   8,
  CHROMA_DC_4x4 =   9,
  CB_16DC       =  10,
  CB_16AC       =  11,
  CB_8x8        =  12,
  CB_8x4        =  13,
  CB_4x8        =  14,
  CB_4x4        =  15,
  CR_16DC       =  16,
  CR_16AC       =  17,
  CR_8x8        =  18,
  CR_8x4        =  19,
  CR_4x8        =  20,
  CR_4x4        =  21
} CABACBlockTypes;

// Macro defines
#define Q_BITS          15
#define DQ_BITS          6
#define Q_BITS_8        16
#define DQ_BITS_8        6 


#define IS_I16MB(MB)    ((MB)->mb_type==I16MB  || (MB)->mb_type==IPCM)
#define IS_DIRECT(MB)   ((MB)->mb_type==0     && (currSlice->slice_type == B_SLICE ))

#define TOTRUN_NUM       15
#define RUNBEFORE_NUM     7
#define RUNBEFORE_NUM_M1  6

// Quantization parameter range
#define MIN_QP          0
#define MAX_QP          51
// 4x4 intra prediction modes 
typedef enum {
  VERT_PRED            = 0,
  HOR_PRED             = 1,
  DC_PRED              = 2,
  DIAG_DOWN_LEFT_PRED  = 3,
  DIAG_DOWN_RIGHT_PRED = 4,
  VERT_RIGHT_PRED      = 5,
  HOR_DOWN_PRED        = 6,
  VERT_LEFT_PRED       = 7,
  HOR_UP_PRED          = 8
} I4x4PredModes;

// 16x16 intra prediction modes
typedef enum {
  VERT_PRED_16   = 0,
  HOR_PRED_16    = 1,
  DC_PRED_16     = 2,
  PLANE_16       = 3
} I16x16PredModes;

// 8x8 chroma intra prediction modes
typedef enum {
  DC_PRED_8     =  0,
  HOR_PRED_8    =  1,
  VERT_PRED_8   =  2,
  PLANE_8       =  3
} I8x8PredModes;

enum {
  EOS = 1,    //!< End Of Sequence
  SOP = 2,    //!< Start Of Picture
  SOS = 3,     //!< Start Of Slice
  SOS_CONT = 4
};

// MV Prediction types
typedef enum {
  MVPRED_MEDIAN   = 0,
  MVPRED_L        = 1,
  MVPRED_U        = 2,
  MVPRED_UR       = 3
} MVPredTypes;

enum {
  DECODING_OK     = 0,
  SEARCH_SYNC     = 1,
  PICTURE_DECODED = 2
};

#define  LAMBDA_ACCURACY_BITS         16
#define INVALIDINDEX  (-135792468)

#define RC_MAX_TEMPORAL_LEVELS   5

//Start code and Emulation Prevention need this to be defined in identical manner at encoder and decoder
#define ZEROBYTES_SHORTSTARTCODE 2 //indicates the number of zero bytes in the short start-code prefix

#define MAX_PLANE       3
#define IS_FREXT_PROFILE(profile_idc) ( profile_idc>=FREXT_HP || profile_idc == FREXT_CAVLC444 )
#define HI_INTRA_ONLY_PROFILE         (((p_Vid->active_sps->profile_idc>=FREXT_Hi10P)&&(p_Vid->active_sps->constrained_set3_flag))||(p_Vid->active_sps->profile_idc==FREXT_CAVLC444)) 

#endif

