
/*!
 *************************************************************************************
 * \file mc_prediction.h
 *
 * \brief
 *    definitions for motion compensated prediction
 *
 * \author
 *      Main contributors (see contributors.h for copyright, 
 *                         address and affiliation details)
 *      - Alexis Michael Tourapis  <alexismt@ieee.org>
 *
 *************************************************************************************
 */

#ifndef _MC_PREDICTION_H_
#define _MC_PREDICTION_H_

#include "global.h"
#include "mbuffer.h"

extern int  allocate_pred_mem(Slice *currSlice);
extern void free_pred_mem    (Slice *currSlice);

extern void get_block_luma(StorablePicture *curr_ref, int x_pos, int y_pos, int hor_block_size, int ver_block_size, imgpel **block,
			                     int shift_x,int maxold_x,int maxold_y,int **tmp_res,int max_imgpel_value,imgpel no_ref_value,Macroblock *currMB);

extern void intra_cr_decoding    (Macroblock *currMB, int yuv);
extern void prepare_direct_params(Macroblock *currMB, StorablePicture *dec_picture, MotionVector *pmvl0, MotionVector *pmvl1,char *l0_rFrame, char *l1_rFrame);

#if NOKIA_USTC_B_VSP_M26069
extern void copy_vsp_pixels(imgpel**input_pixels, imgpel**output_pixels,int start_x,int start_y,int end_x, int end_y, int pad_w,int pad_h);
#endif

#if MTK_SKIP_DIRECT_A0045
extern void prepare_mvp_direct_params(Macroblock *currMB);
extern void set_mvp_direct_params(Macroblock* currMB, MotionVector *pmvl0, MotionVector *pmvl1, char *l0_rFrame, char *l1_rFrame);
#endif

#if(SMRC_3DV && SMRC_EXPLICIT_ALC_4_BSKIP_CHANGERULES_1)
void skip_Bmacroblock_4ALC(Macroblock *currMB, MotionVector *pmvl0, MotionVector *pmvl1, char *l0_rFrame, char *l1_rFrame);
#endif

#if(SMRC_ALC_BFRAMES_ENABLED && SMRC_EXPLICIT_ALC_4_BSKIP_CHANGERULES_1 && SMRC_EXPLICIT_ALC_4_BSKIP_CHANGERULES_1_CONDITIONAL)
int checkNeigborInterviewPred(Macroblock *currMB);
#endif

extern void perform_mc           (Macroblock *currMB, ColorPlane pl, StorablePicture *dec_picture, int pred_dir, int i, int j, int block_size_x, int block_size_y);
#endif

