#include "ATMdecLog4Analise.h"

#if(DECLOG4ANALYSIS)

#define DEFAULT_LOG4ANAL_FILENAME_TEMPLATE	"ADL4A_"

#if(_DUMP_PREDICTORS_)
#define DEFAULT_PRED_FILENAME_TEMPLATE	"pred_"
#endif

#include "mbuffer.h"
#include "global.h"

#define MSK_AF_MERL					0x00000001
#define MSK_AF_ALC					0x00000002

enum MbCodeModeExt
{
    MODE_MVC =       0,
    MODE_FAP =       1,
    MODE_ALC =       2,
    MODE_EFAP =      3,
    MODE_SKIP_FAP =  4,
		MODE_SKIP_MERL = 5,
    MODE_HMST0     = 6,
    MODE_SKIP_ALC	 = 7,
    MODE_SKIP_MERL_ALC = 8
};

// current version
#if (PRINT_QP_VALUE)
#define LAST_VERSION        0xDEAD0A02
#else  // (PRINT_QP_VALUE)
#define LAST_VERSION        0xDEAD0A00
#endif // (PRINT_QP_VALUE)

// past versions

// body

// log header
struct smblogheader
{
    unsigned int size;              // sizeof(this struct) (zero in position indicate old non versioned version of log)
    unsigned int version;           // log version 
};

// frame header
struct smbframeheader
{
    unsigned int size;              // sizeof(this struct)
    unsigned int version;           // struct version 
    unsigned int num_code_order;
    unsigned int widthMB;           // width (in block units, quantity of MB in row)
    unsigned int heigntMB;          // height (in block units, quantity of MB in column)
    unsigned int view;              // = m_IsBaseView ? 1 : 0;
    unsigned int slice_type;
    unsigned int num_disp_order;
    unsigned int lambda;
#if (PRINT_QP_VALUE)
    unsigned int basis_qp;
#endif // (PRINT_QP_VALUE)
};

// macroblock header
struct smbloginfo
{
    unsigned int size;                  //sizeof(this struct)
    unsigned int version;               // struct version
    unsigned char coding_type;           // 0 - mvc; 1... - another

    // standard params
    unsigned char mb_partition_mode;    // mb_type (m_eMbMode)
    unsigned char mb_pred_mode;         // auxillary mb_type (not necessary?)
    unsigned char sub_partition_mode[4];// subblocks types
    unsigned int FwdBwd;                // Pred Mvs existing
    unsigned char Flag8x8;              // 8x8 transform applied
    unsigned int cbp;                   // coded block pattern
    char mb_ref[2][4];                  // P0_Pred,P1_Pred,Bi_Pred derived from this
    short mb_mvs[2][16][2];
    short mb_mvds[2][16][2];
    float mb_cost;                      // mb cost
    int mb_wbits;                       // overall bits
    int mb_tbits;                       // bits for texture (with cbp)
    int mb_mbits;                       // bits for motion (mv + refidx)
    char            IntraLumaPredMode[16];  // Luma intra prediction modes
    unsigned char   IntraChrmPredMode;      // Chroma intra prediction mode

    unsigned int additional_flags;			// 0x0 - MERL
																				// 0x1 - ALC
};
/*
// MV data
struct mvdata
{
	unsigned int mb_partition_mode;    // mb_type (m_eMbMode)
	unsigned int sub_partition_mode[4];// subblocks types
    unsigned int FwdBwd;                // Pred Mvs existing
    unsigned int Flag8x8;              // 8x8 transform applied
    unsigned int cbp;                   // coded block pattern
    int mb_ref[2][4];                  // P0_Pred,P1_Pred,Bi_Pred derived from this
    int mb_mvs[2][16][2];
};
*/
enum CAnalPredMode
{
  CAPM_INTRA        = 0,
  CAPM_INTER        = 1,
  CAPM_BI           = 2,
  CAPM_PCM          = 3,
  CAPM_BI_DIRECT    = 4
};

	static FILE* m_pOutputFile[2][MAX_CODEVIEW];
//	FILE* m_pOutputFileMV;
//	bool  m_IsBaseView;

	static unsigned int m_FrameNumber;

//	unsigned int m_piHeader[LOG4ANAL_HEADER_SIZE];
//	int m_piMvBuffer[2];
  static struct smbloginfo *pMap;
//	mvdata* pMVMap;

	static unsigned int m_BlockAmount;
	static unsigned int m_BlockCount;
  static unsigned int h;
  static unsigned int w;
  static unsigned int bh;
  static unsigned int bw;
  static unsigned int curViewID;
  static unsigned int is_depth;
  static char* prefix;

#if(_DUMP_PREDICTORS_)
	static imgpel** pred_frame = NULL;
	static FILE* m_pOutputPredFile[2][MAX_CODEVIEW];
#endif

int ADL4A_Init( char* h264fn)
{
  prefix = h264fn;
  memset( m_pOutputFile, 0, sizeof(FILE*)*2*MAX_CODEVIEW);

#if(_DUMP_PREDICTORS_)
  memset( m_pOutputPredFile, 0, sizeof(FILE*)*2*MAX_CODEVIEW);
#endif

  //m_pOutputFileMV = 0;
   curViewID = 0;

  m_FrameNumber = 0;

  h = 0;
  w = 0;
  bh = 0;
  bw = 0;
  m_BlockAmount = 0;
  m_BlockCount  = 0;
  pMap = NULL;
  //pMVMap = NULL;

  return 0;
}

void ADL4A_Deinit()
{
  int i;
  /*for( i = 0; i < 2*MAX_CODEVIEW; i++)
	  if (0 != m_pOutputFile[i%2][i/2])
	  {
		  fclose(m_pOutputFile[i%2][i/2]);
		  m_pOutputFile[i%2][i/2] = 0;
	  }*/
/*
	if (0 != m_pOutputFileMV)
	{
		fclose(m_pOutputFileMV);
		m_pOutputFileMV = 0;
	}
*/
  curViewID = 0;
  m_FrameNumber  = 0;

  h = 0;
  w = 0;
  bh = 0;
  bw = 0;
  m_BlockAmount = 0;
  m_BlockCount  = 0;
  if(NULL != pMap)
  {
      free(pMap);
      pMap = NULL;
  }
/*
	if(NULL != pMVMap)
  {
      delete[] pMVMap;
      pMVMap = NULL;
  }
  */
}

//int  ADL4A_Open(  char* prefix, unsigned int ViewID, unsigned int FrameWidth, unsigned int FrameHeight, unsigned int BlockWidth, unsigned int BlockHeight)

int  ADL4A_Open(/*  char* prefix, */Slice *currSlice)
{
  VideoParameters *p_Vid = currSlice->p_Vid;
  char fn[256]={0};
#if(_DUMP_PREDICTORS_)
	char fn_pred[256]={0};
#endif
  struct smblogheader lh;
  unsigned int ViewID = currSlice->view_id;
  unsigned int FrameWidth = p_Vid->width;
  unsigned int FrameHeight = p_Vid->height;
  unsigned int BlockWidth = 16;
  unsigned int BlockHeight = 16;
  //char fnMV[256]={0};
  curViewID = ViewID;

  is_depth = currSlice->is_depth;

  if(is_depth)
    sprintf( fn, "%s%s_v%d.dep.bin", DEFAULT_LOG4ANAL_FILENAME_TEMPLATE, prefix, ViewID);
  else
    sprintf( fn, "%s%s_v%d.tex.bin", DEFAULT_LOG4ANAL_FILENAME_TEMPLATE, prefix, ViewID);

#if(_DUMP_PREDICTORS_)
  if(is_depth)
    sprintf( fn_pred, "%s%s_v%d.dep.yuv", DEFAULT_PRED_FILENAME_TEMPLATE, prefix, ViewID);
  else
    sprintf( fn_pred, "%s%s_v%d.tex.yuv", DEFAULT_PRED_FILENAME_TEMPLATE, prefix, ViewID);
#endif

  if( 0 == m_pOutputFile[is_depth][curViewID])
	{
    m_pOutputFile[is_depth][curViewID] = fopen( fn, "wb");
		#if(_DUMP_PREDICTORS_)
			m_pOutputPredFile[is_depth][curViewID] = fopen( fn_pred, "wb");
		#endif
	}
  else
  {
    m_pOutputFile[is_depth][curViewID] = fopen( fn, "ab");
		#if(_DUMP_PREDICTORS_)
			m_pOutputPredFile[is_depth][curViewID] = fopen( fn_pred, "wb");
		#endif
    h = FrameHeight;
    w = FrameWidth;
    bh = BlockHeight;
    bw = BlockWidth;
    if(m_BlockAmount < ((h+bh-1)/bh)*((w+bw-1)/bw))
    {
      m_BlockAmount =  ((h+bh-1)/bh)*((w+bw-1)/bw);
      if(NULL != pMap)
      {
        free(pMap);
        pMap = (struct smbloginfo*)malloc(sizeof(struct smbloginfo)*m_BlockAmount);
      }
			#if(_DUMP_PREDICTORS_)
				if(NULL != pred_frame)
				{
					free_mem2Dpel( pred_frame);
					get_mem2Dpel( &pred_frame, p_Vid->height, p_Vid->width);
				}
			#endif
    }
    else
      m_BlockAmount =  ((h+bh-1)/bh)*((w+bw-1)/bw);

    return 0;
  }

  if ( m_pOutputFile[is_depth][curViewID] == NULL )
	{
		return (-1);
	}
/*
#ifdef ADV_STAT_ON
	_stprintf( fnMV, "%s%d.bin", DEFAULT_MV4MTLAB_FILENAME_TEMPLATE, ViewID);
	if ( _tfopen_s(&m_pOutputFileMV, fnMV, _TEXT("wb")) )
	{
		return (-1);
	}
#endif
*/

  lh.size = sizeof(struct smblogheader);
  lh.version = LAST_VERSION;
  if( 1 != fwrite( &lh, sizeof(struct smblogheader), 1, m_pOutputFile[is_depth][curViewID]))
  {
      return (-2);
  }

  h = FrameHeight;
  w = FrameWidth;
  bh = BlockHeight;
  bw = BlockWidth;

  m_BlockAmount =  ((h+bh-1)/bh)*((w+bw-1)/bw);
  m_BlockCount  = 0;

#if(_DUMP_PREDICTORS_)
	get_mem2Dpel( &pred_frame, p_Vid->height, p_Vid->width);
#endif

  //pMap = new smbloginfo[m_BlockAmount];
  pMap = (struct smbloginfo*)malloc(sizeof(struct smbloginfo)*m_BlockAmount);
  //pMVMap = new mvdata[m_BlockAmount];

	return 0;
}

int ADL4A_WriteHeader( Slice *currSlice, double lambda)
{
  struct smbframeheader fh;
  float la = (float)lambda;
  int DispOrderNum = currSlice->p_Vid->frame_no;
  int SliceType = currSlice->slice_type ;
  curViewID = currSlice->view_id;
  is_depth = currSlice->is_depth;

	{
		if( 0 != ADL4A_Open( currSlice))
      return -1;
	}

  fh.size = sizeof(struct smbframeheader);
  fh.version = LAST_VERSION;
  fh.num_code_order = m_FrameNumber;
  fh.widthMB =  (w+bw-1)/bw;
  fh.heigntMB = (h+bh-1)/bh;
  fh.view = curViewID;
  fh.slice_type = SliceType;
  fh.num_disp_order = DispOrderNum;
  fh.lambda = *((unsigned int *)&la);

#if (PRINT_QP_VALUE)
  fh.basis_qp = currSlice->qp;
#endif // (PRINT_QP_VALUE)

	m_BlockCount  = 0;

	// write header data
	if ( 1 != fwrite((void*)(&fh), sizeof(struct smbframeheader), 1, m_pOutputFile[is_depth][curViewID]) )
  {
    return (-2);
  }

	fflush(m_pOutputFile[is_depth][curViewID]);

	return 0;
}

#if(SMRC_3DV && (0 != SMRC_SILENT_ALC_4_PSKIP || _DUMP_PREDICTORS_))
void ADL4A_SetMBDopInfo( Macroblock* currMB, double cost, int writtenbits, int texturebits, int mvpredbits, int saveStCodType)
{
	Slice *currSlice = currMB->p_Slice;

#if(0 != SMRC_SILENT_ALC_4_PSKIP )
	if(currMB->mb_type == 0 && (currMB->alc_in_skip_enabled == 1 || currMB->is_alc_mode))
	{
		pMap[m_BlockCount-1].coding_type = MODE_SKIP_ALC;
		#if(MERL_VSP)
		if(currMB->vsskip_flag || currMB->direct_type_flag)
			pMap[m_BlockCount-1].coding_type = MODE_SKIP_MERL_ALC;
		#endif
	}
#endif

#if(_DUMP_PREDICTORS_)
		copy_image_data_16x16(&pred_frame[currMB->pix_y], currSlice->mb_pred[0], currMB->pix_x, 0);
#endif
}
#endif

void ADL4A_SetMBInfo( Macroblock* currMB, double cost, int writtenbits, int texturebits, int mvpredbits, int saveStCodType)
{
  int i,j;
  Slice *currSlice = currMB->p_Slice;
  VideoParameters *p_Vid = currSlice->p_Vid;
  int j4   = currMB->block_x;
	int i4   = currMB->block_y;
	int mb_type_4_SetMBInfo = currMB->mb_type;
	PicMotionParams **mv_info = currSlice->dec_picture->mv_info;

	memset(pMap+m_BlockCount, 0, sizeof(struct smbloginfo));
  pMap[m_BlockCount].size = sizeof(struct smbloginfo);
  pMap[m_BlockCount].version = LAST_VERSION;

	pMap[m_BlockCount].coding_type = 0;

#if(SMRC_3DV)
	if(1 == currMB->is_alc_mode)
	{
		pMap[m_BlockCount].coding_type = MODE_ALC;
		pMap[m_BlockCount].additional_flags |= MSK_AF_ALC;
	}
	#if( (0 != SMRC_SILENT_ALC_4_PSKIP))
	if(currMB->mb_type == 0 && currMB->alc_in_skip_enabled == 1)
		pMap[m_BlockCount].coding_type = MODE_ALC;
	#endif
#endif

#if(MERL_VSP)
	if(currMB->vsskip_flag || currMB->direct_type_flag)
	{
		mb_type_4_SetMBInfo = 0;
		pMap[m_BlockCount].coding_type = MODE_SKIP_MERL;
		pMap[m_BlockCount].additional_flags |= MSK_AF_MERL;
	}
#endif

  pMap[m_BlockCount].mb_ref[0][0] = 0;
  pMap[m_BlockCount].mb_ref[0][1] = 0;
  pMap[m_BlockCount].mb_ref[0][2] = 0;
  pMap[m_BlockCount].mb_ref[0][3] = 0;
  pMap[m_BlockCount].mb_ref[1][0] = 0;
  pMap[m_BlockCount].mb_ref[1][1] = 0;
  pMap[m_BlockCount].mb_ref[1][2] = 0;
  pMap[m_BlockCount].mb_ref[1][3] = 0;

  pMap[m_BlockCount].mb_partition_mode = (unsigned char)mb_type_4_SetMBInfo;//pcMbDataAccess->getMbData().getMbMode();
  pMap[m_BlockCount].FwdBwd = 0;//pcMbDataAccess->getMbData().getFwdBwd();
  pMap[m_BlockCount].mb_cost = (float)cost;
  pMap[m_BlockCount].mb_wbits = currMB->bitlen_full;
  pMap[m_BlockCount].mb_tbits = currMB->bitlen_tex;
  pMap[m_BlockCount].mb_mbits = currMB->bitlen_mot;
  pMap[m_BlockCount].Flag8x8 = currMB->luma_transform_size_8x8_flag;//pcMbDataAccess->getMbData().isTransformSize8x8();
  pMap[m_BlockCount].cbp = currMB->cbp;//pcMbDataAccess->getMbData().getMbExtCbp();

  //Bool  bIsSkip  = pcMbDataAccess->isSkippedMb();
  if( mb_type_4_SetMBInfo == 0)
  {// with direct mode
      //pMap[m_BlockCount].mb_pred_mode = CAPM_SKIP; //skip
      if(currSlice->slice_type == B_SLICE)
      {
          if( currMB->cbp == 0)
              pMap[m_BlockCount].mb_pred_mode = CAPM_BI; // BI SKIP
          else
              pMap[m_BlockCount].mb_pred_mode = CAPM_BI_DIRECT; // BI DIRECT
      }
      else
          pMap[m_BlockCount].mb_pred_mode = CAPM_INTER;  //INTER

      pMap[m_BlockCount].mb_ref[LIST_0][0] = mv_info[i4][j4].ref_idx[0];
      pMap[m_BlockCount].mb_mvds[LIST_0][0][0] = 0;
      pMap[m_BlockCount].mb_mvds[LIST_0][0][1] = 0;
      pMap[m_BlockCount].mb_mvs[LIST_0][0][0] = mv_info[i4][j4].mv[0].mv_y;
      pMap[m_BlockCount].mb_mvs[LIST_0][0][1] = mv_info[i4][j4].mv[0].mv_x;

      pMap[m_BlockCount].mb_ref[LIST_1][0] = mv_info[i4][j4].ref_idx[1];
      pMap[m_BlockCount].mb_mvds[LIST_1][0][0] = 0;
      pMap[m_BlockCount].mb_mvds[LIST_1][0][1] = 0;
      pMap[m_BlockCount].mb_mvs[LIST_1][0][0] = mv_info[i4][j4].mv[1].mv_y;
      pMap[m_BlockCount].mb_mvs[LIST_1][0][1] = mv_info[i4][j4].mv[1].mv_x;

      //memset(pMVMap+m_BlockCount, 0, sizeof(mvdata));
      ++m_BlockCount;
      return;
  }

  if( mb_type_4_SetMBInfo == IPCM)
  {
    //===== PCM SAMPLES =====
      pMap[m_BlockCount].mb_pred_mode = CAPM_PCM; //PCM
  }
  else if( currMB->is_intra_block )
  {
    //===== INTRA PREDICTION MODES =====
      pMap[m_BlockCount].mb_pred_mode = CAPM_INTRA; //INTRA
      if(mb_type_4_SetMBInfo == I16MB)
          pMap[m_BlockCount].IntraLumaPredMode[0] = (char)currMB->i16mode;
      else
      {
          for( i = 0; i < 4; ++i)
            for( j = 0; j < 4; ++j)
              pMap[m_BlockCount].IntraLumaPredMode[i*4+j] = currSlice->ipredmode[i4+i][j4+j];
      }
      pMap[m_BlockCount].IntraChrmPredMode = currMB->c_ipred_mode;
  }
  else
  {   //===== INTER PREDICTION MODES =====
      //===== MOTION INFORMATION =====
    
      if(currSlice->slice_type == B_SLICE)
      {
          pMap[m_BlockCount].mb_pred_mode = CAPM_BI;  //BI
          //WritePredInfo( pcMbDataAccess, (MbMode)pMap[m_BlockCount].mb_partition_mode, LIST_0 );
          //WritePredInfo( pcMbDataAccess, (MbMode)pMap[m_BlockCount].mb_partition_mode, LIST_1 );
      }
      else
      {
          pMap[m_BlockCount].mb_pred_mode = CAPM_INTER;  //INTER
          //WritePredInfo( pcMbDataAccess, (MbMode)pMap[m_BlockCount].mb_partition_mode, LIST_0 );
      }

      for( i = 0; i < 4; ++i)
      {
        pMap[m_BlockCount].mb_ref[LIST_0][i] = mv_info[i4+(i/2)*2][j4+(i%2)*2].ref_idx[0];
        pMap[m_BlockCount].mb_ref[LIST_1][i] = mv_info[i4+(i/2)*2][j4+(i%2)*2].ref_idx[1];
        for( j = 0; j < 4; ++j)
        {
          pMap[m_BlockCount].mb_mvs[LIST_0][i*4+j][0] = mv_info[i4][j4].mv[0].mv_y;
          pMap[m_BlockCount].mb_mvs[LIST_0][i*4+j][1] = mv_info[i4][j4].mv[0].mv_x;
          pMap[m_BlockCount].mb_mvs[LIST_1][i*4+j][0] = mv_info[i4][j4].mv[1].mv_y;
          pMap[m_BlockCount].mb_mvs[LIST_1][i*4+j][1] = mv_info[i4][j4].mv[1].mv_x;

          pMap[m_BlockCount].mb_mvds[LIST_0][i*4+j][0] = currMB->mvd[0][i][j][1];
          pMap[m_BlockCount].mb_mvds[LIST_0][i*4+j][1] = currMB->mvd[0][i][j][0];
          pMap[m_BlockCount].mb_mvds[LIST_1][i*4+j][0] = currMB->mvd[1][i][j][1];
          pMap[m_BlockCount].mb_mvds[LIST_1][i*4+j][1] = currMB->mvd[1][i][j][0];
        }
      }

  }

  if(mb_type_4_SetMBInfo ==P8x8)
      for( i = 0; i < 4; ++i )
              pMap[m_BlockCount].sub_partition_mode[i] = currMB->b8mode[i];

  ++m_BlockCount;
}

int ADL4A_SavePictureData()
{
	// for debug purposes
	_ASSERTE(m_BlockAmount == m_BlockCount);

	if (0 == m_pOutputFile[is_depth][curViewID])
	{
		return (-1);
	}

	// write header data
	if ( m_BlockAmount != fwrite((void*)pMap, sizeof(struct smbloginfo), m_BlockAmount, m_pOutputFile[is_depth][curViewID]) )
	{
			return (-2);
	}

#if(_DUMP_PREDICTORS_)
	fwrite(&(pred_frame[0][0]), sizeof(imgpel)*h*w, 1, m_pOutputPredFile[is_depth][curViewID]);
	fclose(m_pOutputPredFile[is_depth][curViewID]);
#endif

  fclose(m_pOutputFile[is_depth][curViewID]);

	++m_FrameNumber;

	return 0;
}

#else//DECLOG4ANALYSIS

//int ADL4A_Init( char* h264fn){;}
//void ADL4A_Deinit(){;}
//int  ADL4A_Open( Slice *currSlice);
//int ADL4A_WriteHeader( Slice *currSlice, double lambda){;}
//void ADL4A_SetMBInfo( Macroblock* currMB, double cost, int wb, int tb, int mb, int saveStCodType){;}
//int ADL4A_SavePictureData(){;}

#endif//DECLOG4ANALYSIS
