/*!
 **************************************************************************
 * \file defines.h
 *
 * \brief
 *    Header file containing some useful global definitions
 *
 * \author
 *    Main contributors (see contributors.h for copyright, address and affiliation details)
 *     - Detlev Marpe
 *     - Karsten Shring                 <suehring@hhi.de> 
 *     - Alexis Michael Tourapis         <alexismt@ieee.org> 
 *   
 *
 * \date
 *    21. March 2001
 **************************************************************************
 */


#ifndef _DEFINES_H_
#define _DEFINES_H_

#define SMRC_3DV                  1 // !< enables using SMRC tools

#if(SMRC_3DV)
	#include "defs_alc_common.h"

	#define USE_MR_METRIC_FOR_ALC     1 // Turn on Mean-removed metric for ALC inter-view prediction
	#define MR_METRIC_WITH_DC_ADD     1 // if 1 - sum of abs differences of DC or (DC^2) is added to MR_SAD4x4 or MR_SSE4x4
	#define MRSAD_4x4                 1 // !< enables using MRSAD 4x4 function for displacement vector estimation
	#define MRSAD4x4_IMPL             0 // 0 - mike implementation of MRSAD by MRSAD4x4; 1 - alex implementation of MRSAD by MRSAD4x4. Mike implementation includes code for DC addition
	#define ALC_USE_SSE               0

	#define FULL_SEARCH_FOR_ALC       1 // !< enables using Full Search for displacement vector estimation
	#define QPEL_FSEARCH_FOR_ALC      0 // 1 - means full search over all QPEL positions inside search window processes; 0 - means classical "FSearch" - INT-PEL-->H-PEL-->Q-{E:
	#define SMRC_ALC_STFULLSEARCH_AROUND0	0 // 1 - spiral search starts from zero in ALC mode (not from prediction as in standard mode) 
	#define ALC_SET_INTERVIEW_CENTER		1 // 1 - spiral search starts from zero in ALC mode (not from prediction as in standard mode) in "SubPelBlockMotionSearchHighPrecision" function

	#define SMRC_ALC_NO_RESIDUALS_ZEROING			0

	#define FIX_P8x8_RD			0						// 1 - currMB->mb_type is set before solving of RD task for P8x8 mode (this affects the solution of RD task)

	#define INCLINING_2_SKIP	0						// 1 - priority of Skip mode in case of equality of distortion without considering rate values
#endif

#define NOKIA_USTC_3DV            1


#if defined _DEBUG
# define TRACE           0 //1      //!< 0:Trace off 1:Trace on 2:detailed CABAC context information
# define TRACE_RANGE     1 //1    
#define TRACE_RDLOG      1    //!< 0:Trace RD information off 1:Trace RD information on
#define EXT_RDLOG		 1
#else
# define TRACE                    0    //!< 0:Trace off 1:Trace on 2:detailed CABAC context information
# define TRACE_RANGE     0    
#define TRACE_RDLOG      0    //!< 0:Trace RD information off 1:Trace RD information on
#define EXT_RDLOG		 0
#endif

#define DEBUG_PROCESSING	0
#if( SMRC_3DV)
#define SMRC_DUMP_PREDICTORS 0          // allows to dump predictors generated  in different prediction modes
#endif

#if NOKIA_USTC_3DV
#define JM                  "17 (FRExt)"
#define VERSION             "0.4"
#define LOW_PROFILE          1   //!< 0:HP 1:EHP
#if LOW_PROFILE
#define EXT_VERSION         "LOW Profile"
#else
#define EXT_VERSION         "HIGH  Profile"
#endif

#else
#define JM                  "17 (FRExt)"
#define VERSION             "17.2"
#define EXT_VERSION         "(FRExt)"
#endif

#define GET_METIME                1    //!< Enables or disables ME computation time
#define DUMP_DPB                  0    //!< Dump DPB info for debug purposes
#define PRINTREFLIST              0    //!< Print ref list info for debug purposes
#define IMGTYPE                   0   //!< Define imgpel size type. 0 implies byte (cannot handle >8 bit depths) and 1 implies unsigned short
#define ENABLE_FIELD_CTX          1    //!< Enables field context types for CABAC. If disabled, results in speedup for progressive content.
#define ENABLE_HIGH444_CTX        1    //!< Enables High 444 context types for CABAC. If disabled, results in speedup of non High444 profile encodings.
#define DEBUG_BITDEPTH            0    //!< Ensures that > 8 bit content have no values that would result in out of range results
#define ALLOW_GRAYSCALE           1    //!< Allow encoding in grayscale
#define ZEROSNR                   0    //!< PSNR computation method
#define USE_RND_COST              0    //!< Perform ME RD decision using a rounding estimate of the motion cost
#define JM_INT_DIVIDE             1
#define JM_MEM_DISTORTION         0
#define JCOST_CALC_SCALEUP        1    //!< 1: J = (D<<LAMBDA_ACCURACY_BITS)+Lambda*R; 0: J = D + ((Lambda*R+Rounding)>>LAMBDA_ACCURACY_BITS)
#define INTRA_RDCOSTCALC_ET       1    //!< Early termination 
#define INTRA_RDCOSTCALC_NNZ      1    //1: to recover block's nzn after rdcost calculation;
#define JCOST_OVERFLOWCHECK       0    //!<1: to check the J cost if it is overflow>
#define JM_PARALLEL_DEBLOCK       0    //!< Enables Parallel Deblocking


#define EOS_OUTPUT                0 



#if NOKIA_USTC_3DV

//>>>tools belong to  HP & EHP
#define MVC_EXTENSION_ENABLE      0      //!< enable support for the Multiview High Profile
#define MAX_CODEVIEW              16     //!<the max number of coding view
#define MAX_CODEVIEWPLUSDEPTH     32     //!<the max number of coding view plus depth
#define MAX_VIEWREFERENCE          2

#define DEBUG_MVC_PRED_STRUCT      0
#define DEBUG_MMCO                 0
#define DEBUG_RPLR                 0
#define REDUCED_RES                1
#define LANCZOS3          0
#define BI_LINEAR         1

#define IMG_Y             0
#define IMG_U             1
#define IMG_V             2
#define NOKIA_HLS_ADOPTION_M24892   1

#define DEPTH_PARAMETER_SET					1     // JCT2-A0008

//<<< tools belong to  HP & EHP

#if LOW_PROFILE
//>>>tools belong to  EHP
#define MTK_SKIP_DIRECT_A0045       1 //  JCT2-A0045
#define JOINT_VIEW_FILTER           1
#define DRWP                        1     //!<depth range based weight prediction
#define VISBD_VSP                   1
#define DEPTH_BASED_MVP             1
#define GRADUAL_VIEW_REFRESH        1
#define ADAPTIVE_MMCO_REORDER       1
#define MAX_TEMPORAL_LEVELS         6
#define SLICE_HEADER_PREDICTION     1      //!<Slice header prediction from texture to depth, or depth to texture
#define IVMP                        1      //!<Inside View Motion Prediction from texture to depth
#define SONY_MDFY_SEI_AQINF_M24730  1      //!<depth_acquisition_info SEI m24730
#define NTT_MDFY_SEI_DAQINF_A0140              1      //!<depth_acquisition_info SEI modification based on JCT2-A0140
#define POZNAN_NONLINEAR_DEPTH      1      //!<Non-linear depth representation
#define POZNAN_NTT_DEPTH_REPRESENTATION_SEI    1      //!<Depth Representation SEI
#define SAIT_DEPTH_INTRA_PREDICTION 1
#define SAIT_RDO_M23662             1
#define SAIT_POST_DILATION_A0038    1      //!<depth dilation in post processing
#define DRWP                       1   //!<depth range based weight prediction
//<<< tools belong to  EHP
#endif

#if JOINT_VIEW_FILTER
#define PRE_JVF                     1
#define INLOOP_JVF                  1
#define MAX_SIZE_REPORT             1024
#else
#define PRE_JVF                     0
#define INLOOP_JVF                  0
#endif

#if SAIT_RDO_M23662
#define max(a,b)                    (((a) > (b)) ? (a) : (b))
#define min(a,b)                    (((a) < (b)) ? (a) : (b))
#define SAIT_RDO_M24826             1

#if SAIT_RDO_M24826
#define MAX_SYN_VIEWS               64
#define MAX_VIEWS                   64
#endif
#endif

#if SAIT_DEPTH_INTRA_PREDICTION
#define SAIT_DISP                   1
#define SAIT_PSIP                   1
#if SAIT_DISP
#define ETRIKHU_DEBLOCK_IN_DISP     1
#endif
#endif

#if DEPTH_BASED_MVP
#define D_MVP_MODE                    1 //!<0: complex mode, 1:fast mode
#define SAIT_INTER_VIEW_MVP_M24824    1 //!< 1: Enable improved inter-view motion vector prediction. 0: Disable. - Jin Young Lee (Samsung&Sharp)
#define SAIT_DISP_BASED_SAD_M24824    1 //!< 1: Enable disparity-based SAD for skip/direct motion vector competitioin. 0: Disable. - Jin Young Lee
#define SAIT_SKIP_MVP_COMMENT_M24824  1 //!< Editor's comment >> 1: Average disparity for skip inter-view MV. 0: Max disaprity for skip inter-view MV.
#define SAIT_SHIFT_SAD_COMMENT_M24824 1 //!< Editor's comment >> 1: Multiplication for disparity SAD. 0: Shift operation for disparity SAD.
#if SAIT_INTER_VIEW_MVP_M24824
#define MTK_MAX_DV_SIMP_A0046         1 //  JCT2-A0046 
#endif
#endif

#if VISBD_VSP
#define MERL_VSP                    1 //!< 1: Enable adaptive reference for SKIP/DIRECT mode. 0: Disable. -Dong
#define DEBUG_MERL_VSP              0
#if MERL_VSP
#define SAIT_CASFP_M24819           1 //!< 1: Enable context-based adaptive skip flag positioning (CASFP) for view synthesis skip mode. 0: Disable. - Jin Young Lee
#endif
#define NOKIA_HLS_ADOPTION_M24890   1
#define NOKIA_USTC_B_VSP_M26069     1
#if REDUCED_RES
#define QC_VSP_Asymmetric_Resolution  1
#endif
#endif

#if SLICE_HEADER_PREDICTION
#define  FLEXIBLE_PRED_M24946_M24981 1
#endif

#if DRWP
#define NOKIA_USTC_INT_DRWP_M26074   1
#endif

#define SAIT_PSIP_CAVLC_BUGFIX_V5     1    // 1. moving cavlc function (macroblock.x->vlc.x)
#define SAIT_RDO_BUGFIX_V5            1    // 2. support full resolution for VSO (#define REDUCED_RES 0)
#define SAIT_SEI_BUGFIX_V5            1    // 3. support full resolution for SEI (#define REDUCED_RES 0)


#endif

#define DEBUG_MEM_LEAK              0 //!<detect probable memory leak for window platform

#if DEBUG_MEM_LEAK
#define _CRTDBG_MAP_ALLOC
#include <stdlib.h>
#include <crtdbg.h>
#endif



#define MVC_DEVELOPPING           1  

#define EPZSREF                   1

#define MAX_RC_MODE               3
#define RC_MAX_TEMPORAL_LEVELS    5

#define SSE_MEMORY_ALIGNMENT      16

//#define BEST_NZ_COEFF 1   // yuwen 2005.11.03 => for high complexity mode decision (CAVLC, #TotalCoeff)


//landeyan
#define FORCE_YUV400              1
#define FORCE400_OUTPUT420        1
//landeyan


//AVC Profile IDC definitions
enum {
  FREXT_CAVLC444 = 44,       //!< YUV 4:4:4/14 "CAVLC 4:4:4"
  BASELINE       = 66,       //!< YUV 4:2:0/8  "Baseline"
  MAIN           = 77,       //!< YUV 4:2:0/8  "Main"
  EXTENDED       = 88,       //!< YUV 4:2:0/8  "Extended"
  FREXT_HP       = 100,      //!< YUV 4:2:0/8  "High"
  FREXT_Hi10P    = 110,      //!< YUV 4:2:0/10 "High 10"
  FREXT_Hi422    = 122,      //!< YUV 4:2:2/10 "High 4:2:2"
  FREXT_Hi444    = 244,      //!< YUV 4:4:4/14 "High 4:4:4"
  MULTIVIEW_HIGH = 118,      //!< YUV 4:2:0/8  "Multiview High"
  STEREO_HIGH    = 128,       //!< YUV 4:2:0/8  "Stereo High"
#if NOKIA_USTC_3DV
  ThreeDV_HIGH   = 138,
  ThreeDV_EXTEND_HIGH = 139
#endif
} ProfileIDC;

// Some typedefs used in the software
#include "types.h"

#define FILE_NAME_SIZE  255
#define INPUT_TEXT_SIZE 1024

#define CAVLC_LEVEL_LIMIT 2063

#if (ENABLE_HIGH444_CTX == 1)
# define NUM_BLOCK_TYPES 22  
#else
# define NUM_BLOCK_TYPES 10
#endif

///#define _LEAKYBUCKET_

// ---------------------------------------------------------------------------------
// FLAGS and DEFINES for new chroma intra prediction, Dzung Hoang
// Threshold values to zero out quantized transform coefficients.
// Recommend that _CHROMA_COEFF_COST_ be low to improve chroma quality
#define _LUMA_COEFF_COST_       4 //!< threshold for luma coeffs
#define _CHROMA_COEFF_COST_     4 //!< threshold for chroma coeffs, used to be 7
#define _LUMA_MB_COEFF_COST_    5 //!< threshold for luma coeffs of inter Macroblocks
#define _LUMA_8x8_COEFF_COST_   5 //!< threshold for luma coeffs of 8x8 Inter Partition

//#define IMG_PAD_SIZE           20 //!< Number of pixels padded around the reference frame (>=4)
//#define IMG_PAD_SIZE_TIMES4    80 //!< Number of pixels padded around the reference frame in subpel units(>=16)
#define IMG_PAD_SIZE_X         32 //!< Number of pixels padded around the reference frame (>=4)
#define IMG_PAD_SIZE_Y         18  //!< Number of pixels padded around the reference frame (>=4)
#define IMG_PAD_SIZE_X_TIMES4  128 //!< Number of pixels padded around the reference frame in subpel units(>=16)
#define IMG_PAD_SIZE_Y_TIMES4  72 //!< Number of pixels padded around the reference frame in subpel units(>=16)

#define MAX_VALUE       999999   //!< used for start value for some variables
#define INVALIDINDEX  (-135792468)

#define DUMMY   14
#define ET_SIZE 300      //!< size of error text buffer

#define  LAMBDA_ACCURACY_BITS         5
#define  LAMBDA_FACTOR(lambda)        ((int)((double)(1 << LAMBDA_ACCURACY_BITS) * lambda + 0.5))

#if (IMGTYPE == 0)
#define DISTBLK_MAX  INT_MAX
#elif (IMGTYPE == 2)
#define DISTBLK_MAX  FLT_MAX
#else
#define DISTBLK_MAX  ((distblk) INT_MAX << LAMBDA_ACCURACY_BITS)
#endif

#define  MAXSLICEPERPICTURE           100
#define  MAX_REFERENCE_PICTURES        32

#define BLOCK_SHIFT            2
#define BLOCK_SIZE             4
#define BLOCK_SIZE_8x8         8
#define SMB_BLOCK_SIZE         8
#define BLOCK_PIXELS          16
#define MB_BLOCK_SIZE         16
#define MB_PIXELS            256 // MB_BLOCK_SIZE * MB_BLOCK_SIZE
#define MB_PIXELS_SHIFT        8 // log2(MB_BLOCK_SIZE * MB_BLOCK_SIZE)
#define MB_BLOCK_SHIFT         4
#define BLOCK_MULTIPLE         4 // (MB_BLOCK_SIZE/BLOCK_SIZE)
#define MB_BLOCK_PARTITIONS   16 // (BLOCK_MULTIPLE * BLOCK_MULTIPLE)
#define BLOCK_CONTEXT         64 // (4 * MB_BLOCK_PARTITIONS)
// These variables relate to the subpel accuracy supported by the software (1/4)
#define BLOCK_SIZE_SP      16  // BLOCK_SIZE << 2
#define BLOCK_SIZE_8x8_SP  32  // BLOCK_SIZE8x8 << 2

// wavelet based weighted PSNR wavelet levels
#define NUM_WAVELET_LEVEL 4

// RDOQ
#define MAX_PREC_COEFF    25



//  Available MB modes
enum {
  PSKIP        =  0,
  BSKIP_DIRECT =  0,
  P16x16       =  1,
  P16x8        =  2,
  P8x16        =  3,
  SMB8x8       =  4,
  SMB8x4       =  5,
  SMB4x8       =  6,
  SMB4x4       =  7,
  P8x8         =  8,
  I4MB         =  9,
  I16MB        = 10,
  IBLOCK       = 11,
  SI4MB        = 12,
  I8MB         = 13,
  IPCM         = 14,
#if MERL_VSP
  PVSP_SKIP    = 15,
  BVSP_DIRECT  = 15,
#if SAIT_DISP
  DISPMODE     = 16,
  MAXMODE      = 33//17
#else
  MAXMODE      = 16
#endif
#else
#if SAIT_DISP
  DISPMODE     = 15,
  MAXMODE      = 16
#else
  MAXMODE      = 15
#endif
#endif
} MBModeTypes;

// number of intra prediction modes
#define NO_INTRA_PMODE  9

// Direct Mode types
enum {
  DIR_TEMPORAL = 0, //!< Temporal Direct Mode
  DIR_SPATIAL  = 1 //!< Spatial Direct Mode
} DirectModes;

// CAVLC block types
enum {
  LUMA              =  0,
  LUMA_INTRA16x16DC =  1,
  LUMA_INTRA16x16AC =  2,
  CB                =  3,
  CB_INTRA16x16DC   =  4,
  CB_INTRA16x16AC   =  5,
  CR                =  8,
  CR_INTRA16x16DC   =  9,
  CR_INTRA16x16AC   = 10
} CAVLCBlockTypes;

// CABAC block types
enum {
  LUMA_16DC     =   0,
  LUMA_16AC     =   1,
  LUMA_8x8      =   2,
  LUMA_8x4      =   3,
  LUMA_4x8      =   4,
  LUMA_4x4      =   5,
  CHROMA_DC     =   6,
  CHROMA_AC     =   7,
  CHROMA_DC_2x4 =   8,
  CHROMA_DC_4x4 =   9,
  CB_16DC       =  10,
  CB_16AC       =  11,
  CB_8x8        =  12,
  CB_8x4        =  13,
  CB_4x8        =  14,
  CB_4x4        =  15,
  CR_16DC       =  16,
  CR_16AC       =  17,
  CR_8x8        =  18,
  CR_8x4        =  19,
  CR_4x8        =  20,
  CR_4x4        =  21
} CABACBlockTypes;

#if SAIT_DISP
#define IS_INTRA(MB)    ((MB)->mb_type==SI4MB || (MB)->mb_type==I4MB || (MB)->mb_type==I16MB || (MB)->mb_type==I8MB || (MB)->mb_type==IPCM || (MB)->mb_type==DISPMODE)
#else
#define IS_INTRA(MB)    ((MB)->mb_type==SI4MB || (MB)->mb_type==I4MB || (MB)->mb_type==I16MB || (MB)->mb_type==I8MB || (MB)->mb_type==IPCM)
#endif

#define LEVEL_NUM         6
#define TOTRUN_NUM       15
#define RUNBEFORE_NUM     7
#define RUNBEFORE_NUM_M1  6

// Quantization parameter range
#define MIN_QP          0
#define MAX_QP          51
#define SHIFT_QP        12

// 4x4 intra prediction modes 
enum {
  VERT_PRED            = 0,
  HOR_PRED             = 1,
  DC_PRED              = 2,
  DIAG_DOWN_LEFT_PRED  = 3,
  DIAG_DOWN_RIGHT_PRED = 4,
  VERT_RIGHT_PRED      = 5,
  HOR_DOWN_PRED        = 6,
  VERT_LEFT_PRED       = 7,
  HOR_UP_PRED          = 8
} I4x4PredModes;

// 16x16 intra prediction modes
enum {
  VERT_PRED_16   = 0,
  HOR_PRED_16    = 1,
  DC_PRED_16     = 2,
  PLANE_16       = 3
} I16x16PredModes;

// 8x8 chroma intra prediction modes
enum {
  DC_PRED_8     =  0,
  HOR_PRED_8    =  1,
  VERT_PRED_8   =  2,
  PLANE_8       =  3
} I8x8PredModes;

#define INIT_FRAME_RATE 30
enum {
  EOS = 1,    //!< End Of Sequence
  SOP = 2,    //!< Start Of Picture
  SOS = 3     //!< Start Of Slice
};

// MV Prediction types
enum {
  MVPRED_MEDIAN   = 0,
  MVPRED_L        = 1,
  MVPRED_U        = 2,
  MVPRED_UR       = 3
} MVPredTypes;

#define MAX_SYMBOLS_PER_MB  1200  //!< Maximum number of different syntax elements for one MB
                                  // CAVLC needs more symbols per MB


#define MAX_PART_NR     3 /*!< Maximum number of different data partitions.
                               Some reasonable number which should reflect
                               what is currently defined in the SE2Partition map (elements.h) */

//Start code and Emulation Prevention need this to be defined in identical manner at encoder and decoder
#define ZEROBYTES_SHORTSTARTCODE 2 //indicates the number of zero bytes in the short start-code prefix

#define Q_BITS          15
#define DQ_BITS         6

#define Q_BITS_8        16
#define DQ_BITS_8       6

// Context Adaptive Lagrange Multiplier (CALM)
#define CALM_MF_FACTOR_THRESHOLD 512.0

#define MAX_PLANE       3
#define IS_FREXT_PROFILE(profile_idc) ( profile_idc>=FREXT_HP || profile_idc == FREXT_CAVLC444 )

#define MAXSLICEGROUPIDS 8


#define NUM_MB_TYPE_CTX  11
#define NUM_B8_TYPE_CTX  9
#define NUM_MV_RES_CTX   10
#define NUM_REF_NO_CTX   6
#define NUM_DELTA_QP_CTX 4
#define NUM_MB_AFF_CTX   4

#define NUM_TRANSFORM_SIZE_CTX 3

#define NUM_IPR_CTX    2
#define NUM_CIPR_CTX   4
#define NUM_CBP_CTX    4
#define NUM_BCBP_CTX   4
#define NUM_MAP_CTX   15
#define NUM_LAST_CTX  15
#define NUM_ONE_CTX    5
#define NUM_ABS_CTX    5

#if IVMP
#define NUM_IVMP_CTX 3
#endif

#if SAIT_PSIP
#define NUM_PSIP_DIR_CTX           1
#define NUM_PSIP_CODE_DIFF_CTX     2
#endif

#if SAIT_DISP
#define NUM_DISP_CTX  3 
#endif

#endif

