// cavy 2011.02.10
// adopted for ATM 2012.03.12

//#include "H264AVCEncoderLib.h"
//#include "JMVC_CFAP.h"
#include "typedefs.h"
#include "defines.h"
#include "macroblock.h"

#if(TRACE_RDLOG)

    FILE* pfRDLog;
    distblk bestcost;
    int bestbits;
    int bestdist;
    char besttype[128];
    int bestTransType;
    int bestChromaPred;
    char* modeNames[MAXMODE+3] =
    {
      "SKIP     ",//0
      "P16x16   ",//1
      "P16x8    ",//2
      "P8x16    ",//3
      "SMB8x8   ",//4
      "SMB8x4   ",//5
      "SMB4x8   ",//6
      "SMB4x4   ",//7
      "P8x8     ",//8
      "I4MB     ",//9
      "I16MB    ",//10
      "IBLOCK   ",//11
      "SI4MB    ",//12
      "I8MB     ",//13
      "IPCM     ",//14
      "MERL     ",//15 //BVSP_DIRECT
      "DISP     ",//16 //DISPMODE, SAIT
      "ALC_SKIP ",//17      0
      "ALC16x16 ",//18      1
      "ALC16x8  ",//19      2
      "ALC8x16  ",//20      3
      "ALC_S8x8 ",//21      4
      "ALC_S8x4 ",//22      5
      "ALC_S4x8 ",//23      6
      "ALC_S4x4 ",//24      7
      "ALC8x8   ",//25      8
      "X        ",//26      9
      "X        ",//27      10
      "X        ",//28      11
      "X        ",//29      12
      "X        ",//30      13
      "X        ",//31      14
      "ALC_MERL ",//32      15
    };

void CloseRDLog()
{
    if(NULL != pfRDLog)
    {
        fclose(pfRDLog);
        pfRDLog = NULL;
    }
}

int OpenRDLog(char *fn)
{
    if(NULL != pfRDLog)
        CloseRDLog();
    pfRDLog = fopen( fn, "wt");
    if( NULL != pfRDLog)
        return 0;
    else
        return -1;
}

void SetRDCharacteristics(char* type, distblk cost, uint32 bits, uint32 dist, uint32 TransType, uint32 ChromaPred, Macroblock *currMB, int SliceType, int* blk4x4_refs)
{
	const char t4[]= "t4";
	const char t8[]= "t8";
	char* trans;
	char bipred_me = 0;
	char pdir = 0;
	int i;
    if(NULL == pfRDLog)
        return;
    //fprintf( pfRDLog, "      %s cost=% 8d bits=% 8d dist=% 8d\n", type, (int)cost, bits, dist);
#if(!EXT_RDLOG)
		if(TransType)
		  fprintf( pfRDLog, "      %s cost=% 8d bits=% 8d dist=% 8d (ch%d) t8\n", type, (int)cost, bits, dist, ChromaPred);
		else
		  fprintf( pfRDLog, "      %s cost=% 8d bits=% 8d dist=% 8d (ch%d) t4\n", type, (int)cost, bits, dist, ChromaPred);
#else
		if(TransType)
			trans = t8;
		else
			trans = t4;
		bipred_me = currMB->p_Vid->b8x8info->best[1][1].bipred;
		pdir = currMB->p_Vid->b8x8info->best[1][1].pdir;

		fprintf( pfRDLog, "      %s bpred=%d%d cost=% 8d bits=% 8d dist=% 8d (ch%d) %s \tEXT: dist=% 8d CBP_Dq=% 8d luma=% 8d chroma=% 8d MV=% 8d ref=% 8d SliceType=%d Refs: ", 
					type, pdir, bipred_me, (int)cost, bits, dist, ChromaPred, trans, currMB->Statistics_distortion, 
					currMB->Statistics_rate_CBPandDquant, currMB->Statistics_rate_CoeffsLuma, currMB->Statistics_rate_CoeffsChroma, 
					currMB->Statistics_rate_MotionVectors, currMB->Statistics_rate_ReferenceFrame, SliceType);
		for(i =0; i < 16; ++i)
        {
			if (-1 == blk4x4_refs[i])
                fprintf( pfRDLog, "S ");
            else
            if (0 == blk4x4_refs[i])
                fprintf( pfRDLog, "T ");
            else
            if (1 == blk4x4_refs[i])
                fprintf( pfRDLog, "I ");
            else
            if (2 == blk4x4_refs[i])
                fprintf( pfRDLog, "V ");
            else
                fprintf( pfRDLog, "O ");
        }
        fprintf( pfRDLog, "\n");

#endif

		fflush(pfRDLog);
    if(bestcost > cost)
    {
        bestcost = cost;
        bestbits = bits;
        bestdist = dist;
        bestTransType = TransType;
        bestChromaPred = ChromaPred;
        strcpy( besttype, type);
    }
}

void SetRDFrameInfo(uint32 view, uint32 frame, double lyambda, int lyambdaInt, int FactorInt, int Qua)
{
    if(NULL == pfRDLog)
        return;
    fprintf( pfRDLog, "View number = %d; Frame number = %d\n", view, frame);
    fprintf( pfRDLog, "Lambda = %f (Quantizer = %d)\nReal multiplier= %d (L*%d)\n", lyambda, Qua, lyambdaInt, FactorInt);
    fprintf( pfRDLog, "******************************************\n\n", view, frame);
    bestcost = INT_MAX;
}

void EndRDMBInfo()
{
	const char t4[]= "t4";
	const char t8[]= "t8";
	char* trans;
	if(bestTransType)
		trans = t8;
	else
		trans = t4;
    if(NULL == pfRDLog)
        return;
    //fprintf( pfRDLog, "BEST: %s cost=% 8d bits=% 8d dist=% 8d\n", besttype, (int)bestcost, bestbits, bestdist);
    fprintf( pfRDLog, "BEST: %s cost=% 8d bits=% 8d dist=% 8d (ch%d) %s\n", besttype, (int)bestcost, bestbits, bestdist, bestChromaPred, trans);
    

    bestcost = INT_MAX;
    fflush(pfRDLog);
}

void SetRDMBInfo( uint32 mb)
{
    if(NULL == pfRDLog)
        return;
    fprintf( pfRDLog, "mbnum = %d\n", mb);
}

#else//(TRACE_RDLOG)

int OpenRDLog(char *fn)
{
  return 0;
}

void SetRDCharacteristics(char* type, distblk cost, uint32 bits, uint32 dist, uint32 TransType, uint32 ChromaPred)
{}

void SetRDFrameInfo(uint32 view, uint32 frame, double lyambda, int lyambdaInt, int FactorInt, int Qua)
{}

void EndRDMBInfo()
{}

void SetRDMBInfo( uint32 mb)
{}

void CloseRDLog()
{}

#endif//(TRACE_RDLOG)
